# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Action Status
#
# Page d'accueil d'un serveur de commande
#
###########################################################################

""" Actions affichant les différents statuts du serveur """
import xmlrpclib, socket
from os import stat
from os.path import isfile

from zephir.monitor.agentmanager import config as config_agent

from ead2.config.config import dico
from ead2.backend.lib.action import Action, Dict
from ead2.backend.actions.tools import get_request
from ead2.backend.actions.lib.widgets import main as M, ajax
from ead2.backend.config.config import AGENT_PORT, AGENT_PROTOCOL, \
EOLE_MODULE as module
from ead2.backend.config.filenames import bacula, maj, index
from ead2.frontend.web.config import ACTION_SOCKET_TIMEOUT

try:
    from scribe.importation.config import RAPPORTFILE
    IMPORTATION = True
except:
    RAPPORTFILE = ''
    IMPORTATION = False

DOC = ['scribe']

def is_empty(filename):
    """
    vérifie que le fichier existe et qu'il n'est pas vide
    """
    return not isfile(filename) or stat(filename).st_size == 0


class Status(Action):
    """ Action de presentation du statut du serveur
        page d'accueil du serveur
    """
    user_description = Dict(default={}, doc="description de l'exécutant",
                            keys=['ip', 'name', 'role'])
    name = 'main_status'
    libelle = 'Accueil'
    category = ""
    description = "Le statut actuel de votre serveur"
    request = Dict(default={},
                   doc="arguments de la requete en cours cote frontend",
                   keys=['server', 'action', 'aff_maj', 'aff_save',
                         'aff_blacklist', 'aff_extract',
                         'aff_account_list', 'url'])

    def execute(self):
        """ Renvoie les données pour l'affichage
            1 - Renvoie les données pour la mise en page des résumés de
                rapports de sauvegarde, mise à jour, mise à jour de blacklist
            2 - Renvoie les données pour l'affichage complet de rapport
        """
        params, self.server_nb = get_request(self.request)
        result = {}
        toexec = ajax.call(self.server_nb,
                           "rapport_maj",
                           only=False,
                           container="maj_container")
        # présence de eole-bacula
        if dico.has_key('activer_bacula'):
            toexec += ajax.call(self.server_nb,
                                "rapport_sav",
                                only=False,
                                container="bacula_container")
        # présence de eole-proxy
        if dico.has_key('url_maj_blacklist'):
            toexec += ajax.call(self.server_nb,
                                "rapport_blacklist",
                                only=False,
                                container="blackslit_container")
        # présence des librairies scribe
        if IMPORTATION:
            toexec += ajax.call(self.server_nb,
                                "rapport_extraction",
                                only=False,
                                container="extraction_container")
        if module in DOC:
            result.update(self._get_doc_link())
        result.update(self.get_services_status())
        result['toexec'] = toexec
        return self.send_all(result, template='index', templates=['main'])

    def _get_doc_link(self):
        """
            renvoie la description d'une lien vers la documentation
        """
        client_url = self.user_description['ip']
        url = "http://%s/doc-%s/index.html" % (client_url, module)
        href = ajax.popup(url, "Documentation %s" % module, 800, 600)
        return dict(doc_link=M.Bouton(href=href,
                                      icone='/image/help.png',
                                      libelle='Documentation %s' % module,
                                      _class='btn'))

    def get_agent_status(self):
        """
            récupère le status des agents
        """
        serv = xmlrpclib.ServerProxy('%s://localhost:%s/xmlrpc/' % (
                                        AGENT_PROTOCOL, AGENT_PORT))
        try:
            socket.setdefaulttimeout(ACTION_SOCKET_TIMEOUT)
            status = serv.status_for_agents()
            agents_menu = serv.agents_menu()
        except:
            status = {}
            agents_menu = {}
        return status, agents_menu


    def get_services_status(self):
        """ renvoie les données pour la mise en page des status des services
            - les statuts des services
            - la description des boutons de connexion
        """
        result = {}
        if AGENT_PORT is not None:
            status, agents_menu = self.get_agent_status()
            menu_services = {}
            for group, services in agents_menu.items():
                group = group.encode('utf-8')
                options = [[service[0].encode('utf-8'),
                            service[1].encode('utf-8')]for service in services]
                group_status = 'Unknown'
                for service in options:
                    service_status = status.get(service[0], None)
                    if service_status:
                        datas = {}
                        datas['message'] = service_status.get('message',
                                                     "").encode('utf-8')
                        datas['level'] = service_status.get('level',
                                                  '').encode('utf-8')
                        # calcul de l'état du groupe d'agents
                        if datas['level'] == 'OK':
                            if group_status == 'Unknown':
                                group_status = 'OK'
                        elif datas['level'] == 'Warn':
                            if group_status != 'NO':
                                group_status = 'Warn'
                        elif datas['level'] == 'Error':
                            group_status = 'NO'

                        reload(config_agent)
                        client_url = self.user_description['ip']
                        url = '%s://%s:%s/agents/%s/%s/' % (AGENT_PROTOCOL,
                                                       client_url,
                                                       AGENT_PORT,
                                                       config_agent.id_serveur,
                                                       service[0])
                        if 'admin' in self.user_description['role']:
                            datas['href'] = ajax.call(self.server_nb,
                                                      'help',
                                                      agent=url)
                        else:
                            datas['href'] = ''
                        service.append(datas)
                    else:
                        services.remove(service)
                menu_services[group] = {'main_status':group_status,
                                        'options':options}
            return dict(services=menu_services)
        else:
            # pas d'agents Zéphir :(
            return {}

class Rapport(Action):
    """
        Action génrique de renvoie de rapport (sauvegarde, maj, blacklist...)
    """
    user_description = Dict(default={}, doc="description de l'exécutant",
                            keys=['ip', 'name', 'role'])
    name = 'rapport'
    libelle = 'Rapport'
    category = None
    description = "Action générique"
    request = Dict(default={},
                   doc="arguments de la requete en cours cote frontend",
                   keys=['server', 'action', 'aff', 'url'])
    ## configuration
    filename = "/tmp/void"
    html_container = ""
    texte1 = 'Dernier rapport : '
    default_texte = 'Aucun rapport'
    title = "Rapport générique"
    template = "rapport"

    def execute(self):
        """
        """
        params, self.server_nb = get_request(self.request)
        if params.get('aff', None):
            if params.get('aff', ['false'])[0] == 'true':
                datas = self._get_rapport(True)
            else:
                datas = dict(btn=self._get_btn(False))
                datas['rapport'] = []
            return self.send_frag(datas,
                                  template=self.template,
                                  templates=["main"])
        result = dict(title=self.title)
        result.update(self._get_rapport())
        result.update(dict(html_container=self.html_container))
        return self.send_all(result,
                             template=self.template,
                             templates=['main'])

    def _get_rapport(self, aff=False):
        """
            gestion de l'affichage d'un rapport
        """
        if is_empty(self.filename):
            return dict(rapport_resume=[self.texte1,
                                        self.default_texte],
                        diode="",
                        btn="")
        elif aff:
            return self._get_rapport_complet()
        else:
            return self._get_rapport_resume()
        return dict()

    def _get_rapport_complet(self):
        """
            renvoie le rapport complet
        """
        fic = file(self.filename, 'r')
        rapport = fic.readlines()
        fic.close()
        return dict(rapport=rapport,
                    btn=self._get_btn(True))

    def _get_rapport_resume(self):
        """
            renvoie la première ligne du rapport
        """
        fic = file(self.filename, 'r')
        rapport = fic.readlines()
        fic.close()
        diode = self._get_diode(rapport)
        if len(rapport) >= 1:
            rapport = rapport[0]
        return dict(rapport_resume=[self.texte1, rapport],
                    btn=self._get_btn(False),
                    diode=diode)

    def _get_diode(self, rapport):
        """
            renvoie les descriptions pour les diodes de statut
        """
        return ""

    def _get_btn(self, aff=False):
        """
            renvoie la description pour la mise en forme des boutons
        """
        if aff:
            href = ajax.call(self.server_nb,
                             self.name,
                             container=self.html_container,
                             aff='false')
            href += ajax.top()
            return M.Bouton(href=href,
                            icone='/image/reduce.jpg',
                            libelle='Masquer le rapport',
                            _class='btn')
        else:
            href = ajax.call(self.server_nb,
                             self.name,
                             container=self.html_container,
                             aff='true')
            return M.Bouton(href=href,
                            icone='/image/increase.jpg',
                            libelle='Afficher le rapport',
                            _class='btn')

class RapportBacula(Rapport):
    """
        Rapport de sauvegarde
    """
    name = 'rapport_sav'
    libelle = "Rapport de sauvegarde"
    description  = "Rapport de sauvegarde"
    ## configuration
    filename = bacula['rapport']
    # en 2.3 : plus de résumé :(
    rapport_resume = bacula['rapport']
    html_container = "save_report_container"
    texte1 = 'Dernière sauvegarde : '
    default_texte = 'Aucune sauvegarde'
    title = "Sauvegarde"

    def _get_rapport(self, aff=False):
        """
            renvoie le rapport
            aff:rapport complet (True) / rapport résumé (False)
        """
        if 'non' in [dico['activer_bacula'], dico['activer_bacula_dir']]:
            return dict(rapport_resume=["Dernière sauvegarde : ",
   '<i>Les sauvegardes sont désactivées dans le dictionnaire</i>'],
                        btn='',
                        diode='')
        if aff:
            if is_empty(self.filename):
                return dict(rapport_resume=[self.default_texte],
                            diode="",
                            btn="")
            else:
                return self._get_rapport_complet()
        else:
            if is_empty(self.rapport_resume):
                return dict(rapport_resume=[self.texte1, self.default_texte],
                            diode="",
                            btn="")
            else:
                print "appel au résumé"
                return self._get_rapport_resume()
        return dict()

    def _get_rapport_resume(self):
        """
            renvoie un rapport résumé
        """
        fic = file(self.rapport_resume, 'r')
        rapport = fic.readlines()
        fic.close()
        date = rapport[0].replace('-', ' ').split(' ')
        resume = ["Sauvegarde du %s %s à %s" % (date[0], date[1], date[2])]
        diode = "No"
        for rline in rapport:
            if "Backup OK" in rline:
                diode = "OK"
                break
        return dict(rapport_resume=resume,
                    btn=self._get_btn(),
                    diode=diode)

class RapportMaj(Rapport):
    """
        Rapport de mise à jour
    """
    name = 'rapport_maj'
    libelle = "Rapport de mise à jour"
    description  = "Rapport de mise à jour"
    ## configuration
    filename = maj['rapport']
    html_container = "maj_report_container"
    texte1 = 'Dernière mise à jour : '
    default_texte = 'Aucune mise à jour'
    title = "Mise à jour"

    def _get_diode(self, rapport):
        for line in rapport:
            if "Mise à jour OK" in line:
                return "OK"
        return "No"

class RapportBlacklist(Rapport):
    """
        Rapport de mise à jour de listes de sites interdits
    """
    name = 'rapport_blacklist'
    libelle = "Mise à jour des listes de sites interdits"
    description  = "Rapport de mise à jour des listes de sites interdits"
    ## configuration
    filename = index['blacklist']
    html_container = "blacklist_report_container"
    texte1 = 'Dernière mise à jour de la liste de sites interdits : '
    default_texte = 'Aucune mise à jour'
    title = "Liste de sites interdits"


class RapportExtraction(Rapport):
    """
        Rapport d'importation de comptes utilisateurs
    """
    name = 'rapport_extraction'
    libelle = "Rapport d'extraction"
    description  = "Rapport d'extraction"
    filename = RAPPORTFILE
    html_container = "extraction_report_container"
    texte1 = 'Dernière importation : '
    default_texte = 'Aucune importation'
    title = 'Importation'
    template = "scribe_rapport_extraction"

    def _get_rapport_complet(self):
        """
            rapport complet
        """
        fic = file(self.filename, 'r')
        rapport = fic.readlines()
        fic.close()
        return dict(rapport=rapport,
                    btn=self._get_btn(True))

    def _get_rapport_resume(self):
        """
            résumé (1ère ligne du rapport)
        """
        fic = file(self.filename, 'r')
        rapport = fic.readlines()
        fic.close()
        diode = self._get_diode(rapport)
        if len(rapport) >= 1:
            rapport = rapport[0]
        return dict(rapport_resume=[self.texte1, rapport],
                    btn=self._get_btn(False),
                    diode=diode)

