# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2009
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# outils pour Bacula
#
###########################################################################

""" Outils permettant de gérer les sauvegardes bacula """
from os.path import isfile

from pyeole.bacula import load_bacula_support, load_bacula_mail, DAY_TO_STRING, \
        bacula_active_dir, bacula_active_sd, Disabled

#from ead2.config.config import dico as dico_eole
from ead2.backend.config.config import BACULA_DIR, EOLE_MODULE as module
from ead2.backend.actions.tools import command_statusoutput
from ead2.backend.config.filenames import bacula as baculafiles
from ead2.backend.actions.lib.widgets import form as F

# templates communs
USED_TEMPLATES = ['main', 'formselect', 'form']

# fichiers de définition des supports bacula
#FILENAMES = {'usb':baculafiles['usb'],
#             'smb':baculafiles['smb'],
#             'bande':'/etc/bacula/bacula-sd.conf'}

#def get_actual_support():
#    """ renvoie le type de support actuel """
#    code, retour = command_statusoutput('%sbaculasupport.py -l' % BACULA_DIR)
#    if retour in ['smb' , 'usb', 'bande']:
#        return retour
#    else:
#        return ''

def get_bacula_form(pagename):
    """
        renvoie le formulaire adequat
    """
    form = {'level': _level(pagename)}
    if pagename in ('monthly', 'weekly', 'daily'):
        form['begin'] =  _begin_end(name='jourdebut', default='2', pagename=pagename)
        form['hour'] = _hour()
    if pagename == 'daily':
        form['end'] = _begin_end(name='jourfin', default='6', pagename='end')
    return form

def _level(pagename):
    """
        Renvoie la balise pour la sélection du niveau de mise à jour
    """
    # niveau de mise à jour
    level = F.Select(name='niveau', libelle="type", inline=True)
    level.add_option('Full', libelle='Totale')
    if pagename != 'monthly':
        level.add_option('Differential', libelle='Différentielle')
        level.add_option('Incremental', libelle='Incrémentale')
    return level

def _begin_end(name, default, pagename=None):
    # choix du jour
    if pagename == 'daily':
        message='de'
    elif pagename == 'end':
        message='à'
    else:
        message='dans'
    ret = F.Select(name=name, libelle='{0} la nuit du'.format(message), inline=True)
    for day, libelle in DAY_TO_STRING.items():
        ret.add_option(str(day), libelle=libelle, group='nuit de la semaine')
    ret.default(default, group='nuit de la semaine')
    return ret

def _hour():
    """
        Choix de l'heure
    """
    hour = F.Select(name='heure', libelle='à', inline=True)
    hours = range(12, 24)
    hours.extend(range(12))
    for i in hours:
        hour.add_option(str(i),
                        libelle="%d heure" % i,
                        group='heure de la journée')
    hour.default('21', group='heure de la journée')
    return hour

def get_days(_day=None):
    """
        renvoie un dico jouranglais:jourfrancais
        accompagné par un tuple décrivant l'ordre
        :day: jour en anglais (dans ce cas ne renvoie que le jour
                                                     en français)
    """
    days = (('mon', 'lundi'), ('tue', 'mardi'),
            ('wed', 'mercredi'), ('thu', 'jeudi'),
            ('fri', 'vendredi'), ('sat', 'samedi'),
            ('sun', 'dimanche'))
    if _day:
        for day, jour in days:
            if day == _day:
                return day, jour
    else:
        return days

#def bacula_active():
#    """
#        Teste si bacula est activé
#    """
#    return dico_eole['activer_bacula'] == 'oui'
#
#def bacula_active_sd():
#    return 'oui' == dico_eole['activer_bacula_sd']
#
#def bacula_active_dir():
#    return 'oui' == dico_eole['activer_bacula_dir']

def bacula_configured():
    """
        Teste si bacula est configuré
    """
    if bacula_active_sd():
        sup = load_bacula_support()
        if sup['support'] == 'none':
            return False
    if bacula_active_dir():
        sup = load_bacula_mail()
        if not load_bacula_mail:
            return False
    return True

#def get_typesav(typesav=None):
#    """ renvoie un dico décrivant les type de sauvegarde
#        ou la traduction de l'appelation système"""
#    dico = {'Complete': 'Complète', 'Restreinte': 'Restreinte'}
#    if typesav != None:
#        return dico[typesav]
#    return dico

def get_niveau(niveau=None):
    """ renvoie un dico : {'niveau': 'libelle'}
        ou la traduction de l'appelation système"""
    dico = {'Level=Incremental':'Incrémentale', 'Level=Full':'Totale', 'Level=Differential':'Differentielle' }
    dico2 = {'Incremental':'Incrémentale', 'Full':'Totale', 'Differential':'Differentielle'}
    if niveau in dico.keys():
        return dico[niveau]
    elif niveau in dico2.keys():
        return dico2[niveau]
    else:
        return dico
